#
# Verifies the existence of the required jar files and constructs the classpath
# needed for both the SetupFrame and the InitialIndexer applications.
#

# the path to the property file; if they reside elsewhere, change the next line
PROPERTYFILEDIR=../WEB-INF
# Path to jar files; if they reside elsewhere, change the next line
JARFILEDIR=../WEB-INF/lib
# Path to JDBC driver jar file; if this resides elsewhere or you'd like
# to use another one, change the next line
JDBCDRIVERJAR=$JARFILEDIR/mysql.jar
# Paths to Xerces jar files; if they reside elsewhere or you'd like
# to use other ones, change the next line(s). Note: the Xerces version must
# be at least 2.1.0!
XERCESJAR1=$JARFILEDIR/xercesImpl.jar
XERCESJAR2=$JARFILEDIR/xmlParserAPIs.jar
# If you're using another servlet runner than Tomcat or Catalina, change the next line.
# Either you download Tomcat or Catalina and extract servlet.jar into any directory
# given below or you locate the servlet API jar of your servlet runner and its path
# to next line.
# Notes:
# - If you run Tomcat or Catalina, make sure that TOMCAT_HOME or CATALINA_HOME are
#   set, pointing to the installation directory of either of them.
# - If you set SERVLETJAR, the value must include the jar file name!
SERVLETJAR=
# The look'n'feel library used by the setup application.
KUNSTSTOFFJAR=$JARFILEDIR/kunststoff.jar


if [ ! -x "$PROPERTYFILEDIR" ]; then
  echo "  The path you provided for the property file directory isn't valid!"
  exit 1
fi

if [ ! -x "$JARFILEDIR" ]; then
  echo "  The path you provided for the FileLister jar file directory isn't valid!"
  exit 1
fi

if [ ! -r "$KUNSTSTOFFJAR" ]; then
  echo "  The path you provided for the Kunststoff look'n'feel jar file directory isn't valid!"
  exit 1
fi

if [ ! -r "$JDBCDRIVERJAR" ]; then
  echo "  The path you provided for the JDBC driver jar file isn't valid!"
  exit 1
fi

if [ ! -r "$XERCESJAR1" ]; then
  echo "  The path you provided for the first Xerces jar file isn't valid!"
  exit 1
fi

if [ ! -r "$XERCESJAR2" ]; then
  echo "  The path you provided for the second Xerces jar file isn't valid!"
  exit 1
fi

if [ ! -z $SERVLETJAR ]; then
  if [ ! -r "$SERVLETJAR" ]; then
    echo "  The path you provided for the serlvet jar isn't valid!"
    exit 1
  fi
elif [ ! "$CATALINA_HOME" = "" ]; then
# Look for CATALINA_HOME environment variable; if it exists, get Catalina's servlet.jar
  SERVLETJAR=$CATALINA_HOME/common/lib/servlet.jar
  if [ ! -r "$SERVLETJAR" ]; then
    echo "  CATALINA_HOME is set but Catalina's directory structure seems to be invalid!"
    echo "  Do you have a working Catalina installation?"
    exit 1
  fi
elif [ ! "$TOMCAT_HOME" = "" ]; then
# Look for TOMCAT_HOME environment variable; if it exists, get Tomcat's servlet.jar
  SERVLETJAR=$TOMCAT_HOME/lib/common/servlet.jar
  if [ ! -r "$SERVLETJAR" ]; then
    echo "  TOMCAT_HOME is set but Tomcat's directory structure seems to be invalid!"
    echo "  Do you have a working Tomcat installation?"
    exit 1
  fi
else
  echo "  Can't find any servlet jar! Please install Tomcat or Catalina or update the"
  echo "  SERVLETJAR variable within the filelister_env.sh!"
  exit 1
fi

# Set the full classpath for FileLister's applications and return it
FILELISTERCLASSPATH=$JARFILEDIR/filelister-0.5.1.jar
FILELISTERCLASSPATH=$FILELISTERCLASSPATH:$JARFILEDIR/util-1.1.jar
FILELISTERCLASSPATH=$FILELISTERCLASSPATH:$JARFILEDIR/kunststoff.jar
FILELISTERCLASSPATH=$FILELISTERCLASSPATH:$XERCESJAR1
FILELISTERCLASSPATH=$FILELISTERCLASSPATH:$XERCESJAR2
FILELISTERCLASSPATH=$FILELISTERCLASSPATH:$SERVLETJAR
FILELISTERCLASSPATH=$FILELISTERCLASSPATH:$JDBCDRIVERJAR
