package org.alltimeflashdreamer.filelister;

import javax.servlet.ServletException;

/**
 * <p>
 *
 * Title: FileListerException</p> <p>
 *
 * Description: Comments the application-specific errors that may occur during program
 * execution. All the error messages are hard-coded within this class.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerException extends ServletException
{
    //predefined exception texts
    private final static String USEORIGINALINPUTSITE = "<br>Make sure you're using the original input site.";
    private final static String ENTERVALIDDIRECTORY = "<br>Please enter a valid and accessible directory to be searched.";
    private final static String JDBCENVIRONMENT = "<br>Please make sure the JDBC environment is configured correctly.";
    private final static String ABUSE = "<br>Make sure you're not abusing FileLister.";
    private final static String PROPERTYFILEERROR = "<br>Please correct the configuration.";
    private final static String REINDEX = "<br>Please try again and/or rerun the indexing service.";
    private final static String VERIFYINPUT = "<br>Please verify your input.";

    private final static String[] TEXTS =
            {
    //0 - 5
            "File listing input parameter is missing!" + USEORIGINALINPUTSITE,
            "File listing input tells that neither the files nor the directories shall be listed!" + USEORIGINALINPUTSITE,
            "Search mode input parameter is missing!" + USEORIGINALINPUTSITE,
            "Search mode input tells that neither the archive nor the directory mode shall be used!" + USEORIGINALINPUTSITE,
            "The URL parameter \"selectedforzip\" has been modified illegally!" + ABUSE,
            "The virtual root directory isn't valid (either not existing or no directory)!",
    //6 - 10
            "The output site couldn't be generated by DimDom!<br>Please make sure you're using the original site template.",
            "No search configuration has been done yet!<br>Please make sure you're starting with FileLister's configuration site.",
            "A security violation occurred while reading the contents of a directory!<br>Please make sure that the directory to be searched doesn't contain inaccessible files or directories.",
            "The JDBC driver class couldn't be loaded!" + JDBCENVIRONMENT,
            "The archived data couldn't be read from the database!" + JDBCENVIRONMENT,
    //11 - 15
            "The input parameters are illegal!" + ABUSE,
            "Illegal value for the \"jump to site\" input!<br>Please make sure you enter a valid integer > 0.",
            "The index aging parameter in the config file is missing or not a positive number!",
            "The initial archiving process hasn't been done yet!<br>Please run the setup process correctly.",
            "A database query threw an exception!" + JDBCENVIRONMENT,
    //16 - 20
            "The inputted directory doesn't exist!" + ENTERVALIDDIRECTORY,
            "The entered directory is not in the scope!" + ENTERVALIDDIRECTORY,
            "The initial connection, the connection increment or the max. file entry count value is not defined or not a positive number!" + PROPERTYFILEERROR,
            "The connection pool couldn't initialize connections!<br>Please read the exception text below carefully.",
            "A found file or directory couldn't be stored in the database!" + JDBCENVIRONMENT,
    //21 - 25
            "The creation or preparation of a Statement threw an exception!" + JDBCENVIRONMENT,
            "The initializing for the indexing process threw an unexpected exception!",
            "The entered search string isn't a valid regular expression!",
            "A critical error occurred!<br>Please contact the system administrator immediately!",
            "An i/o error occurred: Cannot determine whether it's a symbolic link or not.",
    //26 - 30
            "An i/o error occurred while trying to resolve the list of directories specified in \"directory.nogoarea\".",
            "Missing database table name parameter!" + JDBCENVIRONMENT,
            "The initial indexing process failed!",
            "The database-specific query builder class couldn't be instantiated!<br>Please make sure this class is on the classpath and implements the SQLQueryBuilder interface correctly.",
            "Not specified, application-critical error occurred!",
    //31 - 35
            "Link prefix parameter is invalid! Please verify that the link.prefix parameter in the config file is a valid beginning of an URL.",
            "Some ambiguous error occurred.\nPlease try to restart your servlet runner and/or rerun the indexing service.",
            "The connection pool hasn't been initialized yet!" + ABUSE,
            "The database entry representing the manually searched/reindexed directory couldn't be updated!" + JDBCENVIRONMENT,
            "The list of file numbers to be downloaded is invalid!" + ABUSE,
    //36 - 40
            "File or directory to be added to the download zip is missing or inaccessible!" + REINDEX,
            "A file couldn't be added to the download zip due to an i/o error!" + REINDEX,
            "List of files to be zipped and/or FileListerConfig are missing or misconfigured!" + ABUSE,
            "The opening or the closing of the new zip file lead to an i/o error!" + REINDEX,
            "The directory specified to be the zip directory is missing or not a writable directory!" + PROPERTYFILEERROR,
    //41 - 45
            "The application couldn't create the zip file or couldn't send it to your browser!",
            "No file or directory was selected to be added to the download zip file!",
            "The size multiplier parameter was modified illegally!" + ABUSE,
            "The size input is not a whole-numbered, positive value!" + VERIFYINPUT,
            "The date input is not a valid, parseable date!" + VERIFYINPUT,
    //46 - 50
            "The zip file can't be streamed because it hasn't been / couldn't be created yet or the file was already downloaded!",
            "The config file couldn't be parsed. Please make sure you use the original config file delivered with the distribution!" + ABUSE,
            "Input format doesn't fit the required format of the property!" + VERIFYINPUT,
            "The config file couldn't be found!\nPlease correct the value of the \"filelister.configdir\" parameter in the web.xml file!"
            };

    /**
     * Calls the superclass' constructor with the text referenced by iTextIndex.
     *
     * @param iTextIndex  the index of the text to be shown, as defined with this
     *      class
     */
    public FileListerException( int iTextIndex )
    {
        super( TEXTS[iTextIndex] );
    }

    /**
     * Calls the superclass' constructor with the text referenced by iTextIndex and
     * the given Throwable.
     *
     * @param iTextIndex  the index of the text to be shown, as defined with this
     *      class
     * @param t           the Throwable to be stored by this exception
     */
    public FileListerException( int iTextIndex, Throwable t )
    {
        super( TEXTS[iTextIndex], t );
    }
}
