package org.alltimeflashdreamer.filelister;

import java.sql.SQLException;

import org.alltimeflashdreamer.util.jdbc.ConnectionPool;

/**
 * <p>
 *
 * Title: PoolHandler</p> <p>
 *
 * Description: This simple class provides the (static) access to a single instance
 * of ConnectionPool.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public final class PoolHandler
{
    private static ConnectionPool pool;

//----------------------------------------- private methods ------------------------------------//
    /** Static class, so don't initialize it! */
    private PoolHandler() { }

    /**
     * Returns the pool (if first call, it's initialized first which may throw a
     * "embedded" FileListerException).
     *
     * @return                      the ConnectionPool instance to use
     * @throws FileListerException  if the pool is null / not yet initialized
     */
    public static ConnectionPool getPool()
        throws FileListerException
    {
        if ( pool == null )
            throw new FileListerException( 33 );

        return pool;
    }

    /**
     * Initializes the connection pool according to the settings stored in the given
     * PropertyFileParameters instance, i.e. the property file.
     *
     * @param params                where to get the settings from
     * @throws FileListerException  wrapped ClassNotFoundException or SQLException,
     *      in case something went wrong
     */
    static void initializePool( PropertyFileParameters params )
        throws FileListerException
    {
        if ( pool != null )
            return;

        try
        {
            //initialize connection pool
            pool = new ConnectionPool( params.getConnectionString(), params.getUserName(), params.getPassword(),
                    params.getDriverClassName(), params.getInitialConnections( true ), params.getConnectionIncrement( true ) );
        } catch ( ClassNotFoundException cnfe )
        {
            throw new FileListerException( 9, cnfe );
        } catch ( SQLException sqle )
        {
            throw new FileListerException( 19, sqle );
        }
    }
}
