package org.alltimeflashdreamer.filelister.gui;

import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.alltimeflashdreamer.filelister.xml.FileListerXMLHandler;

/**
 * <p>
 *
 * Title: PropertyFileChooser</p> <p>
 *
 * Description: This file chooser is designed to find the FileLister's property file.
 * </p> <p>
 *
 * Organization: All Time Flash Dreamer</p> <p>
 *
 * </p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class PropertyFileChooser extends JFileChooser
{
    /** Initializes the file chooser with title, buttons and file filter. */
    public PropertyFileChooser()
    {
        setCurrentDirectory( new File( "." ) );
        setDialogTitle( "Select the location of FileLister's config file:" );
        setApproveButtonText( "Select" );
        setApproveButtonMnemonic( KeyEvent.VK_S );
        setApproveButtonToolTipText( "Select the config file" );
        setAcceptAllFileFilterUsed( false );
        setFileFilter( new PropertyFileFilter() );
    }

//----------------------------- inner class PropertyFileFilter ---------------------//
    private class PropertyFileFilter extends FileFilter
    {
        /** empty constructor */
        private PropertyFileFilter() { }

        /**
         * Returns true <i>only if</i> the file name equals "filelister.properties".
         *
         * @param file  the file to check
         * @return      true only if as described above
         */
        public boolean accept( File file )
        {
            return ( file.isDirectory() || file.getName().equals( FileListerXMLHandler.CONFIGFILE ) );
        }

        /**
         * Returns a simple description of the accepted file name.
         *
         * @return   a description for the filter
         */
        public String getDescription()
        {
            return "FileLister's config file (" + FileListerXMLHandler.CONFIGFILE + ")";
        }
    }
}
