package org.alltimeflashdreamer.filelister.gui;

import com.incors.plaf.kunststoff.KunststoffLookAndFeel;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.plaf.metal.MetalLookAndFeel;
import org.alltimeflashdreamer.filelister.*;
import org.alltimeflashdreamer.filelister.xml.FileListerXMLHandler;
import org.alltimeflashdreamer.util.*;
import org.xml.sax.SAXException;

/**
 * <p>
 *
 * Title: SetupFrame</p> <p>
 *
 * Description: </p> <p>
 *
 * Organization: All Time Flash Dreamer</p> <p>
 *
 * </p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class SetupFrame extends JFrame
{
    /** color to be used within the GUI windows (lighter) */
    public final static Color BGCOLOR_1 = new Color( 240, 240, 240 );
    /** color to be used within the GUI windows (darker) */
    public final static Color BGCOLOR_2 = new Color( 230, 230, 230 );
    /** font to be used within the GUI windows (plain) */
    public final static Font FONT_PLAIN = new Font( "Arial", Font.PLAIN, 12 );
    /** font to be used within the GUI windows (bold) */
    public final static Font FONT_BOLD = new Font( "Dialog", Font.BOLD, 12 );

    private final static String SAVE = "Save";
    private final static String SAVEANYWAY = "Save anyway";
    private final static String RESTORE = "Restore";
    private final static String LOADDEFAULTS = "Load Defaults";
    private final static String EXIT = "Exit";
    private final static String TESTCONNECTION = "Test Connection";
    //private final static String RUNINITIALINDEXER = "Run InitialIndexer";

    private JMenuBar jmbMenubar = new JMenuBar();
    private JMenu jmActions = new JMenu( "Actions" );
    private JMenuItem jmiSave = new JMenuItem( SAVE, KeyEvent.VK_S );
    private JMenuItem jmiRestore = new JMenuItem( RESTORE, KeyEvent.VK_R );
    private JMenuItem jmiLoadDefaults = new JMenuItem( LOADDEFAULTS, KeyEvent.VK_D );
    private JMenuItem jmiConnectionTest = new JMenuItem( TESTCONNECTION, KeyEvent.VK_T );
    //private JMenuItem jmiRunInitialIndexer = new JMenuItem( RUNINITIALINDEXER, KeyEvent.VK_I );
    private JMenuItem jmiExit = new JMenuItem( EXIT, KeyEvent.VK_E );
    private FileListerTable jtProperties;
    private FileListerTextArea jtDescriptions;
    private FileListerTableModel model;
    private String sPropertyFileDir = "../";
    private File propertyFile;

    /**
     * The constructor tries to initialize the table model first to find out whether
     * the property file is findable. If so, the GUI is built upon the model, if
     * not, a JFileChooser is shown telling the user that he should select the location
     * of the property file. The initial used path is just "..", assuming the filelister
     * jar file resides in the lib folder of the WEB-INF directory of the servlet
     * runner. If it isn't there, the resulting FileNotFoundException is thrown up
     * to the SetupFrame.
     *
     * @param sPropertyFileDir  the directory where the property file is stored
     */
    public SetupFrame( String sPropertyFileDir )
    {
        propertyFile = new File( StringUtils.appendFileSeparator( sPropertyFileDir, true ) + FileListerXMLHandler.CONFIGFILE );
        //DEBUG: System.out.println( "Assumed property file path: " + propertyFile.getAbsolutePath() );

        loadPropertyFileAndInitializeModel();

        try
        {
            jbInit();
        } catch ( Exception e )
        {
            e.printStackTrace();
        }

        //finally, show itself
        this.show();
    }

    /**
     * Simply calls the constructor.
     *
     * @param args  if empty, they are ignored; if length == 1, then the first argument
     *      is taken as the property file directory
     */
    public static void main( String[] args )
    {
        try
        {
            //set the Look'n'Feel
            UIManager.setLookAndFeel( new KunststoffLookAndFeel() );
        } catch ( UnsupportedLookAndFeelException ulfe )
        {
            try
            {
                UIManager.setLookAndFeel( new MetalLookAndFeel() );
            } catch ( UnsupportedLookAndFeelException ulfe2 )
            {
                ulfe2.printStackTrace();
            }
        }

        String sPropertyFileDir = "../WEB-INF";
        if ( args != null && args.length > 0 )
            sPropertyFileDir = args[0];

        //run the thing
        new SetupFrame( sPropertyFileDir );
    }

//--------------------------------------- inner methods ----------------------------//
    /**
     * GUI builder method.
     *
     * @throws Exception  wraps any exception
     */
    private void jbInit()
        throws Exception
    {
        //set frame icon
        this.setIconImage( ImageUtils.getImageFromFileOrFromJar( this.getClass(),
                "./filelister-" + FileLister.VERSIONNUMBER + ".jar", "setupgui.gif" ) );
        this.setBackground( BGCOLOR_2 );
        this.setResizable( false );
        this.setSize( 550, 450 );
        this.setLocation( 150, 100 );
        this.setTitle( "FileLister v" + FileLister.VERSIONNUMBER + ": Configuration Window" );
        this.addWindowListener(
            new WindowAdapter()
            {
                /**
                 * Exits the virtual machine silently.
                 *
                 * @param event  the window event to handle
                 */
                public void windowClosing( WindowEvent event )
                {
                    closeApplication();
                }
            } );

        this.setJMenuBar( jmbMenubar );
        jmbMenubar.add( jmActions );
        jmActions.add( jmiSave );
        jmActions.add( jmiRestore );
        jmActions.add( jmiLoadDefaults );
        jmActions.addSeparator();
        jmActions.add( jmiConnectionTest );
        //jmActions.add( jmiRunInitialIndexer );
        jmActions.addSeparator();
        jmActions.add( jmiExit );
        jmActions.setMnemonic( 'A' );
        jmiSave.addActionListener(
            new ActionListener()
            {
                /**
                 * Stops the editing, if still on, and stores the configuration,
                 * if's been edited.
                 *
                 * @param event  the action event to handle
                 */
                public void actionPerformed( ActionEvent event )
                {
                    try
                    {
                        jtProperties.editingCanceled( new ChangeEvent( this ) );
                        if ( jtProperties.isEdited() )
                        {
                            model.getXMLHandler().save( model );
                            jtProperties.setEdited( false );
                            reloadConfigFile( false );
                        }
                    } catch ( IOException ioe )
                    {
                        showErrorMessage( "Couldn't save the configuration due to an unexpected i/o error:\n" +
                                ioe.getMessage(), false );
                    } catch ( FileListerException fle )
                    {
                        showErrorMessage( "Couldn't save the configuration due to an illegal change in the configuration:\n" +
                                fle.getMessage() + ( ( fle.getRootCause() != null ) ? "\n" + fle.getRootCause().getMessage() : "" ),
                                false );
                        //reloadConfigFile( false );
                    }
                }
            } );
        jmiSave.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_S, KeyEvent.CTRL_MASK ) );
        jmiRestore.addActionListener(
            new ActionListener()
            {
                /**
                 * Forces the reloading of the config file.
                 *
                 * @param event  the action event to handle
                 */
                public void actionPerformed( ActionEvent event )
                {
                    reloadConfigFile( false );
                }
            } );
        jmiRestore.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_R, KeyEvent.CTRL_MASK ) );
        jmiLoadDefaults.addActionListener(
            new ActionListener()
            {
                /**
                 * Forces the reloading of the config file (with default values).
                 *
                 * @param event  the action event to handle
                 */
                public void actionPerformed( ActionEvent event )
                {
                    reloadConfigFile( true );
                }
            } );
        jmiLoadDefaults.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_D, KeyEvent.CTRL_MASK ) );
        jmiConnectionTest.addActionListener(
            new ActionListener()
            {
                /**
                 * Tries to establish a connection based on the config settings and
                 * comments the upcoming exceptions, in case.
                 *
                 * @param event  the action event to handle
                 */
                public void actionPerformed( ActionEvent event )
                {
                    if ( jtProperties.isEdited() )
                    {
                        JOptionPane.showMessageDialog( SetupFrame.this, "Please save the configuration prior to \n" +
                                "running the connection test!", "Configuration unsaved", JOptionPane.INFORMATION_MESSAGE );
                    }
                    else
                    {
                        ConnectionTesterDialog ctd = new ConnectionTesterDialog(
                                model.getXMLHandler().getJDBC(), getLocation(), getSize() );
                        ctd.show();
                        ctd.requestFocus();
                        jtProperties.editingCanceled( new ChangeEvent( this ) );
                    }
                }
            } );
        jmiConnectionTest.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_T, KeyEvent.CTRL_MASK ) );
        //jmiRunInitialIndexer.addActionListener( this );
        //jmiRunInitialIndexer.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_I, KeyEvent.CTRL_MASK ) );
        jmiExit.addActionListener(
            new ActionListener()
            {
                /**
                 * Exits the VM silently.
                 *
                 * @param event  the action event to handle
                 */
                public void actionPerformed( ActionEvent event )
                {
                    closeApplication();
                }
            } );
        jmiExit.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_F4, KeyEvent.ALT_MASK ) );

        jtProperties = new FileListerTable( model, ( jtDescriptions = new FileListerTextArea() ) );

        this.getContentPane().setLayout( new GridBagLayout() );
        this.getContentPane().add( new JScrollPane( jtProperties ), new GridBagConstraints( 0, 0, 1, 1, 1.0, 0.8,
                GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
        this.getContentPane().add( new JScrollPane( jtDescriptions ), new GridBagConstraints( 0, 1, 1, 1, 1.0, 0.2,
                GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
    }

    /**
     * Shows the given error message in a JOptionPane (type error) and exits the
     * VM, if the given boolean is true.
     *
     * @param sErrorMessage   the error message to be shown.
     * @param bIncludingExit  whether to shut down the VM too or not
     */
    private void showErrorMessage( String sErrorMessage, boolean bIncludingExit )
    {
        JOptionPane.showMessageDialog( this, sErrorMessage, "Configuration error", JOptionPane.ERROR_MESSAGE );
        if ( bIncludingExit )
            System.exit( 1 );
    }

    /**
     * Loads the property file. If this fails, the application is exited. If this
     * succeeds, the table model is initialized using these values.
     */
    private void loadPropertyFileAndInitializeModel()
    {
        if ( !propertyFile.exists() )
        {
            PropertyFileChooser fileChooser = new PropertyFileChooser();
            int iSelection = fileChooser.showDialog( null, "Select" );
            if ( iSelection == JFileChooser.CANCEL_OPTION )
                showErrorMessage( "Setup doesn't work without the config file.\nAction aborted.", true );
            else if ( iSelection == JFileChooser.APPROVE_OPTION )
                propertyFile = fileChooser.getSelectedFile();
        }

        //if the property file is found, the model can be set to the table or the model can be reloaded
        try
        {
            model = new FileListerTableModel( propertyFile );
        } catch ( IOException ioe )
        {
            showErrorMessage( "Unexpected i/o error occurred while reading the file.\nPlease make sure " +
                    "that it's accessible for the setup process!", true );
        } catch ( SAXException saxe )
        {
            showErrorMessage( "The config file couldn't be parsed.\nPlease make sure you use the original " +
                    "config file delivered with the distribution!", true );
        }
    }

    /**
     * Reloads the property file. The given boolean indicates whether to load the
     * default of the user values.
     *
     * @param bLoadDefaults  load default values?
     */
    private void reloadConfigFile( boolean bLoadDefaults )
    {
        try
        {
            model.reload( propertyFile, bLoadDefaults );
            jtProperties.reloadRenderer();
        } catch ( IOException ioe )
        {
            showErrorMessage( "Unexpected i/o error occurred while reloading the file.\n" +
                    "Please make sure that it's accessible for the setup process!", true );
        } catch ( SAXException saxe )
        {
            showErrorMessage( "The config file couldn't be parsed.\nPlease make sure you use the original " +
                    "config file delivered with the distribution!", true );
        }
    }

    /**
     * Shows a question box in case the values remain unsaved. If the user confirms
     * the question, the application is closed without saving of the values, else
     * the closing is canceled.
     */
    private void closeApplication()
    {
        if ( jtProperties.isEdited() )
        {
            int iUserSelection = JOptionPane.showConfirmDialog( this, "The configuration has changed!\n" +
                    "Are you sure you want to exit?", "Configuration unsaved", JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE );
            if ( iUserSelection == JOptionPane.YES_OPTION )
                System.exit( 0 );
        }
        else
            System.exit( 0 );
    }
}
